# encoding: utf-8
from __future__ import absolute_import

import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models
from django.db import DatabaseError


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Adding model 'TaskMeta'
        db.create_table('celery_taskmeta', (
                ('id',self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('task_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
                ('status', self.gf('django.db.models.fields.CharField')(default='PENDING', max_length=50)),
                ('result', self.gf('djcelery.picklefield.PickledObjectField')(default=None, null=True)),
                ('date_done', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
                ('traceback', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),))
        db.send_create_signal('djcelery', ['TaskMeta'])

        # Adding model 'TaskSetMeta'
        db.create_table('celery_tasksetmeta', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('taskset_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
                ('result', self.gf('djcelery.picklefield.PickledObjectField')()),
                ('date_done', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),))
        db.send_create_signal('djcelery', ['TaskSetMeta'])

        # Adding model 'IntervalSchedule'
        db.create_table('djcelery_intervalschedule', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('every', self.gf('django.db.models.fields.IntegerField')()),
                ('period', self.gf('django.db.models.fields.CharField')(max_length=24)),))
        db.send_create_signal('djcelery', ['IntervalSchedule'])

        # Adding model 'CrontabSchedule'
        db.create_table('djcelery_crontabschedule', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('minute', self.gf('django.db.models.fields.CharField')(default='*', max_length=64)),
                ('hour', self.gf('django.db.models.fields.CharField')(default='*', max_length=64)),
                ('day_of_week', self.gf('django.db.models.fields.CharField')(default='*', max_length=64)),))
        db.send_create_signal('djcelery', ['CrontabSchedule'])

        # Adding model 'PeriodicTasks'
        db.create_table('djcelery_periodictasks', (
                ('ident', self.gf('django.db.models.fields.SmallIntegerField')(default=1, unique=True, primary_key=True)),
                ('last_update', self.gf('django.db.models.fields.DateTimeField')()),))
        db.send_create_signal('djcelery', ['PeriodicTasks'])

        # Adding model 'PeriodicTask'
        db.create_table('djcelery_periodictask', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=200)),
                ('task', self.gf('django.db.models.fields.CharField')(max_length=200)),
                ('interval', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['djcelery.IntervalSchedule'], null=True, blank=True)),
                ('crontab', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['djcelery.CrontabSchedule'], null=True, blank=True)),
                ('args', self.gf('django.db.models.fields.TextField')(default='[]', blank=True)),
                ('kwargs', self.gf('django.db.models.fields.TextField')(default='{}', blank=True)),
                ('queue', self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True)),
                ('exchange', self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True)),
                ('routing_key', self.gf('django.db.models.fields.CharField')(default=None, max_length=200, null=True, blank=True)),
                ('expires', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
                ('enabled', self.gf('django.db.models.fields.BooleanField')(default=True)),
                ('last_run_at', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
                ('total_run_count', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
                ('date_changed', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),))
        db.send_create_signal('djcelery', ['PeriodicTask'])

        # Adding model 'WorkerState'
        db.create_table('djcelery_workerstate', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('hostname', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
                ('last_heartbeat', self.gf('django.db.models.fields.DateTimeField')(null=True, db_index=True)),))
        db.send_create_signal('djcelery', ['WorkerState'])

        # Adding model 'TaskState'
        db.create_table('djcelery_taskstate', (
                ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('state', self.gf('django.db.models.fields.CharField')(max_length=64, db_index=True)),
                ('task_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=36)),
                ('name', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, db_index=True)),
                ('tstamp', self.gf('django.db.models.fields.DateTimeField')(db_index=True)),
                ('args', self.gf('django.db.models.fields.TextField')(null=True)),
                ('kwargs', self.gf('django.db.models.fields.TextField')(null=True)),
                ('eta', self.gf('django.db.models.fields.DateTimeField')(null=True)),
                ('expires', self.gf('django.db.models.fields.DateTimeField')(null=True)),
                ('result', self.gf('django.db.models.fields.TextField')(null=True)),
                ('traceback', self.gf('django.db.models.fields.TextField')(null=True)),
                ('runtime', self.gf('django.db.models.fields.FloatField')(null=True)),
                ('retries', self.gf('django.db.models.fields.IntegerField')(default=0)),
                ('worker', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['djcelery.WorkerState'], null=True)),
                ('hidden', self.gf('django.db.models.fields.BooleanField')(default=False, db_index=True)),))
        db.send_create_signal('djcelery', ['TaskState'])


    def backwards(self, orm):

        # Deleting model 'TaskMeta'
        db.delete_table('celery_taskmeta')

        # Deleting model 'TaskSetMeta'
        db.delete_table('celery_tasksetmeta')

        # Deleting model 'IntervalSchedule'
        db.delete_table('djcelery_intervalschedule')

        # Deleting model 'CrontabSchedule'
        db.delete_table('djcelery_crontabschedule')

        # Deleting model 'PeriodicTasks'
        db.delete_table('djcelery_periodictasks')

        # Deleting model 'PeriodicTask'
        db.delete_table('djcelery_periodictask')

        # Deleting model 'WorkerState'
        db.delete_table('djcelery_workerstate')

        # Deleting model 'TaskState'
        db.delete_table('djcelery_taskstate')


    models = {
        'djcelery.crontabschedule': {
            'Meta': {'object_name': 'CrontabSchedule'},
            'day_of_week': ('django.db.models.fields.CharField', [], {'default': "'*'", 'max_length': '64'}),
            'hour': ('django.db.models.fields.CharField', [], {'default': "'*'", 'max_length': '64'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'minute': ('django.db.models.fields.CharField', [], {'default': "'*'", 'max_length': '64'})
        },
        'djcelery.intervalschedule': {
            'Meta': {'object_name': 'IntervalSchedule'},
            'every': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'period': ('django.db.models.fields.CharField', [], {'max_length': '24'})
        },
        'djcelery.periodictask': {
            'Meta': {'object_name': 'PeriodicTask'},
            'args': ('django.db.models.fields.TextField', [], {'default': "'[]'", 'blank': 'True'}),
            'crontab': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['djcelery.CrontabSchedule']", 'null': 'True', 'blank': 'True'}),
            'date_changed': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'enabled': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'exchange': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'interval': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['djcelery.IntervalSchedule']", 'null': 'True', 'blank': 'True'}),
            'kwargs': ('django.db.models.fields.TextField', [], {'default': "'{}'", 'blank': 'True'}),
            'last_run_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'}),
            'queue': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'routing_key': ('django.db.models.fields.CharField', [], {'default': 'None', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'task': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'total_run_count': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        'djcelery.periodictasks': {
            'Meta': {'object_name': 'PeriodicTasks'},
            'ident': ('django.db.models.fields.SmallIntegerField', [], {'default': '1', 'unique': 'True', 'primary_key': 'True'}),
            'last_update': ('django.db.models.fields.DateTimeField', [], {})
        },
        'djcelery.taskmeta': {
            'Meta': {'object_name': 'TaskMeta', 'db_table': "'celery_taskmeta'"},
            'date_done': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('djcelery.picklefield.PickledObjectField', [], {'default': 'None', 'null': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'PENDING'", 'max_length': '50'}),
            'task_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'traceback': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'djcelery.tasksetmeta': {
            'Meta': {'object_name': 'TaskSetMeta', 'db_table': "'celery_tasksetmeta'"},
            'date_done': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('djcelery.picklefield.PickledObjectField', [], {}),
            'taskset_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'djcelery.taskstate': {
            'Meta': {'ordering': "['-tstamp']", 'object_name': 'TaskState'},
            'args': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'eta': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'expires': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'hidden': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'kwargs': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'db_index': 'True'}),
            'result': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'retries': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'runtime': ('django.db.models.fields.FloatField', [], {'null': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '64', 'db_index': 'True'}),
            'task_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '36'}),
            'traceback': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'tstamp': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True'}),
            'worker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['djcelery.WorkerState']", 'null': 'True'})
        },
        'djcelery.workerstate': {
            'Meta': {'ordering': "['-last_heartbeat']", 'object_name': 'WorkerState'},
            'hostname': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_heartbeat': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'db_index': 'True'})
        }
    }

    complete_apps = ['djcelery']



from __future__ import absolute_import

SOUTH_ERROR_MESSAGE = """
For South support, customize the SOUTH_MIGRATION_MODULES setting
to point to the correct migrations module:

    SOUTH_MIGRATION_MODULES = {
        'djcelery': 'djcelery.south_migrations',
    }
"""

try:
    from django.db import migrations  # noqa
except ImportError:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured(SOUTH_ERROR_MESSAGE)



from __future__ import absolute_import, unicode_literals

from functools import wraps

from django.http import HttpResponse, Http404

from anyjson import serialize

from celery import states
from celery.registry import tasks
from celery.result import AsyncResult
from celery.utils import get_full_cls_name, kwdict
from celery.utils.encoding import safe_repr

# Ensure built-in tasks are loaded for task_list view
import celery.task  # noqa


def JsonResponse(response):
    return HttpResponse(serialize(response), content_type='application/json')


def task_view(task):
    """Decorator turning any task into a view that applies the task
    asynchronously. Keyword arguments (via URLconf, etc.) will
    supercede GET or POST parameters when there are conflicts.

    Returns a JSON dictionary containing the keys ``ok``, and
        ``task_id``.

    """

    def _applier(request, **options):
        kwargs = kwdict(request.method == 'POST' and
                        request.POST or request.GET)
        # no multivalue
        kwargs = dict(((k, v) for k, v in kwargs.iteritems()), **options)
        result = task.apply_async(kwargs=kwargs)
        return JsonResponse({'ok': 'true', 'task_id': result.task_id})

    return _applier


def apply(request, task_name):
    """View applying a task.

    **Note:** Please use this with caution. Preferably you shouldn't make this
        publicly accessible without ensuring your code is safe!

    """
    try:
        task = tasks[task_name]
    except KeyError:
        raise Http404('apply: no such task')
    return task_view(task)(request)


def is_task_successful(request, task_id):
    """Returns task execute status in JSON format."""
    return JsonResponse({'task': {
        'id': task_id,
        'executed': AsyncResult(task_id).successful(),
    }})


def task_status(request, task_id):
    """Returns task status and result in JSON format."""
    result = AsyncResult(task_id)
    state, retval = result.state, result.result
    response_data = dict(id=task_id, status=state, result=retval)
    if state in states.EXCEPTION_STATES:
        traceback = result.traceback
        response_data.update({'result': safe_repr(retval),
                              'exc': get_full_cls_name(retval.__class__),
                              'traceback': traceback})
    return JsonResponse({'task': response_data})


def registered_tasks(request):
    """View returning all defined tasks as a JSON object."""
    return JsonResponse({'regular': tasks.regular().keys(),
                         'periodic': tasks.periodic().keys()})


def task_webhook(fun):
    """Decorator turning a function into a task webhook.

    If an exception is raised within the function, the decorated
    function catches this and returns an error JSON response, otherwise
    it returns the result as a JSON response.


    Example:

    .. code-block:: python

        @task_webhook
        def add(request):
            x = int(request.GET['x'])
            y = int(request.GET['y'])
            return x + y

        def view(request):
            response = add(request)
            print(response.content)

    Gives::

        "{'status': 'success', 'retval': 100}"

    """

    @wraps(fun)
    def _inner(*args, **kwargs):
        try:
            retval = fun(*args, **kwargs)
        except Exception as exc:
            response = {'status': 'failure', 'reason': safe_repr(exc)}
        else:
            response = {'status': 'success', 'retval': retval}

        return JsonResponse(response)

    return _inner



# -*- coding: utf-8 -*-
# Generated by Django 1.10.dev20151005181923 on 2015-10-09 22:39
from __future__ import absolute_import, unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import djcelery.picklefield


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CrontabSchedule',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('minute', models.CharField(
                    default='*', max_length=64, verbose_name='minute')),
                ('hour', models.CharField(
                    default='*', max_length=64, verbose_name='hour')),
                ('day_of_week', models.CharField(
                    default='*', max_length=64, verbose_name='day of week')),
                ('day_of_month', models.CharField(
                    default='*', max_length=64, verbose_name='day of month')),
                ('month_of_year', models.CharField(
                    default='*', max_length=64, verbose_name='month of year')),
            ],
            options={
                'ordering': [
                    'month_of_year', 'day_of_month',
                    'day_of_week', 'hour', 'minute'],
                'verbose_name': 'crontab',
                'verbose_name_plural': 'crontabs',
            },
        ),
        migrations.CreateModel(
            name='IntervalSchedule',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('every', models.IntegerField(verbose_name='every')),
                ('period', models.CharField(
                    choices=[
                        ('days', 'Days'),
                        ('hours', 'Hours'),
                        ('minutes', 'Minutes'),
                        ('seconds', 'Seconds'),
                        ('microseconds', 'Microseconds')],
                    max_length=24, verbose_name='period')),
            ],
            options={
                'ordering': ['period', 'every'],
                'verbose_name': 'interval',
                'verbose_name_plural': 'intervals',
            },
        ),
        migrations.CreateModel(
            name='PeriodicTask',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('name', models.CharField(
                    help_text='Useful description', max_length=200,
                    unique=True, verbose_name='name')),
                ('task', models.CharField(
                    max_length=200, verbose_name='task name')),
                ('args', models.TextField(
                    blank=True, default='[]',
                    help_text='JSON encoded positional arguments',
                    verbose_name='Arguments')),
                ('kwargs', models.TextField(
                    blank=True, default='{}',
                    help_text='JSON encoded keyword arguments',
                    verbose_name='Keyword arguments')),
                ('queue', models.CharField(
                    blank=True, default=None,
                    help_text='Queue defined in CELERY_QUEUES',
                    max_length=200, null=True, verbose_name='queue')),
                ('exchange', models.CharField(
                    blank=True, default=None, max_length=200,
                    null=True, verbose_name='exchange')),
                ('routing_key', models.CharField(
                    blank=True, default=None, max_length=200,
                    null=True, verbose_name='routing key')),
                ('expires', models.DateTimeField(
                    blank=True, null=True, verbose_name='expires')),
                ('enabled', models.BooleanField(
                    default=True, verbose_name='enabled')),
                ('last_run_at', models.DateTimeField(
                    blank=True, editable=False, null=True)),
                ('total_run_count', models.PositiveIntegerField(
                    default=0, editable=False)),
                ('date_changed', models.DateTimeField(
                    auto_now=True)),
                ('description', models.TextField(
                    blank=True, verbose_name='description')),
                ('crontab', models.ForeignKey(
                    blank=True,
                    help_text='Use one of interval/crontab', null=True,
                    on_delete=django.db.models.deletion.CASCADE,
                    to='djcelery.CrontabSchedule', verbose_name='crontab')),
                ('interval', models.ForeignKey(
                    blank=True, null=True,
                    on_delete=django.db.models.deletion.CASCADE,
                    to='djcelery.IntervalSchedule', verbose_name='interval')),
            ],
            options={
                'verbose_name': 'periodic task',
                'verbose_name_plural': 'periodic tasks',
            },
        ),
        migrations.CreateModel(
            name='PeriodicTasks',
            fields=[
                ('ident', models.SmallIntegerField(
                    default=1, primary_key=True,
                    serialize=False, unique=True)),
                ('last_update', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='TaskMeta',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('task_id', models.CharField(
                    max_length=255, unique=True,
                    verbose_name='task id')),
                ('status', models.CharField(
                    choices=[
                        (b'RECEIVED', b'RECEIVED'),
                        (b'RETRY', b'RETRY'),
                        (b'REVOKED', b'REVOKED'),
                        (b'SUCCESS', b'SUCCESS'),
                        (b'STARTED', b'STARTED'),
                        (b'FAILURE', b'FAILURE'),
                        (b'PENDING', b'PENDING')],
                    default=b'PENDING', max_length=50,
                    verbose_name='state')),
                ('result', djcelery.picklefield.PickledObjectField(
                    default=None, editable=False, null=True)),
                ('date_done', models.DateTimeField(
                    auto_now=True, verbose_name='done at')),
                ('traceback', models.TextField(
                    blank=True, null=True, verbose_name='traceback')),
                ('hidden', models.BooleanField(
                    db_index=True, default=False, editable=False)),
                ('meta', djcelery.picklefield.PickledObjectField(
                    default=None, editable=False, null=True)),
            ],
            options={
                'db_table': 'celery_taskmeta',
                'verbose_name': 'task state',
                'verbose_name_plural': 'task states',
            },
        ),
        migrations.CreateModel(
            name='TaskSetMeta',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('taskset_id', models.CharField(
                    max_length=255, unique=True,
                    verbose_name='group id')),
                ('result', djcelery.picklefield.PickledObjectField(
                    editable=False)),
                ('date_done', models.DateTimeField(
                    auto_now=True, verbose_name='created at')),
                ('hidden', models.BooleanField(
                    db_index=True, default=False, editable=False)),
            ],
            options={
                'db_table': 'celery_tasksetmeta',
                'verbose_name': 'saved group result',
                'verbose_name_plural': 'saved group results',
            },
        ),
        migrations.CreateModel(
            name='TaskState',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('state', models.CharField(
                    choices=[
                        (b'RECEIVED', b'RECEIVED'),
                        (b'RETRY', b'RETRY'),
                        (b'REVOKED', b'REVOKED'),
                        (b'SUCCESS', b'SUCCESS'),
                        (b'STARTED', b'STARTED'),
                        (b'FAILURE', b'FAILURE'),
                        (b'PENDING', b'PENDING')],
                    db_index=True, max_length=64, verbose_name='state')),
                ('task_id', models.CharField(
                    max_length=36, unique=True, verbose_name='UUID')),
                ('name', models.CharField(
                    db_index=True, max_length=200,
                    null=True, verbose_name='name')),
                ('tstamp', models.DateTimeField(
                    db_index=True, verbose_name='event received at')),
                ('args', models.TextField(
                    null=True, verbose_name='Arguments')),
                ('kwargs', models.TextField(
                    null=True, verbose_name='Keyword arguments')),
                ('eta', models.DateTimeField(
                    null=True, verbose_name='ETA')),
                ('expires', models.DateTimeField(
                    null=True, verbose_name='expires')),
                ('result', models.TextField(
                    null=True, verbose_name='result')),
                ('traceback', models.TextField(
                    null=True, verbose_name='traceback')),
                ('runtime', models.FloatField(
                    help_text='in seconds if task succeeded',
                    null=True, verbose_name='execution time')),
                ('retries', models.IntegerField(
                    default=0, verbose_name='number of retries')),
                ('hidden', models.BooleanField(
                    db_index=True, default=False, editable=False)),
            ],
            options={
                'ordering': ['-tstamp'],
                'get_latest_by': 'tstamp',
                'verbose_name': 'task',
                'verbose_name_plural': 'tasks',
            },
        ),
        migrations.CreateModel(
            name='WorkerState',
            fields=[
                ('id', models.AutoField(
                    auto_created=True, primary_key=True,
                    serialize=False, verbose_name='ID')),
                ('hostname', models.CharField(
                    max_length=255, unique=True, verbose_name='hostname')),
                ('last_heartbeat', models.DateTimeField(
                    db_index=True, null=True, verbose_name='last heartbeat')),
            ],
            options={
                'ordering': ['-last_heartbeat'],
                'get_latest_by': 'last_heartbeat',
                'verbose_name': 'worker',
                'verbose_name_plural': 'workers',
            },
        ),
        migrations.AddField(
            model_name='taskstate',
            name='worker',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to='djcelery.WorkerState', verbose_name='worker'),
        ),
    ]



# -- XXX This module must not use translation as that causes
# -- a recursive loader import!
from __future__ import absolute_import, unicode_literals

from datetime import datetime

from django.conf import settings

# Database-related exceptions.
from django.db import DatabaseError
try:
    import MySQLdb as mysql
    _my_database_errors = (mysql.DatabaseError,
                           mysql.InterfaceError,
                           mysql.OperationalError)
except ImportError:
    _my_database_errors = ()      # noqa
try:
    import psycopg2 as pg
    _pg_database_errors = (pg.DatabaseError,
                           pg.InterfaceError,
                           pg.OperationalError)
except ImportError:
    _pg_database_errors = ()      # noqa
try:
    import sqlite3
    _lite_database_errors = (sqlite3.DatabaseError,
                             sqlite3.InterfaceError,
                             sqlite3.OperationalError)
except ImportError:
    _lite_database_errors = ()    # noqa
try:
    import cx_Oracle as oracle
    _oracle_database_errors = (oracle.DatabaseError,
                               oracle.InterfaceError,
                               oracle.OperationalError)
except ImportError:
    _oracle_database_errors = ()  # noqa

DATABASE_ERRORS = ((DatabaseError, ) +
                   _my_database_errors +
                   _pg_database_errors +
                   _lite_database_errors +
                   _oracle_database_errors)


try:
    from django.utils import timezone
    is_aware = timezone.is_aware

    # see Issue #222
    now_localtime = getattr(timezone, 'template_localtime', timezone.localtime)

    def make_aware(value):
        if getattr(settings, 'USE_TZ', False):
            # naive datetimes are assumed to be in UTC.
            if timezone.is_naive(value):
                value = timezone.make_aware(value, timezone.utc)
            # then convert to the Django configured timezone.
            default_tz = timezone.get_default_timezone()
            value = timezone.localtime(value, default_tz)
        return value

    def make_naive(value):
        if getattr(settings, 'USE_TZ', False):
            default_tz = timezone.get_default_timezone()
            value = timezone.make_naive(value, default_tz)
        return value

    def now():
        if getattr(settings, 'USE_TZ', False):
            return now_localtime(timezone.now())
        else:
            return timezone.now()

except ImportError:
    now = datetime.now

    def _pass(x):
        return x
    make_aware = make_naive = _pass

    def is_aware(x):
        return False


def maybe_make_aware(value):
    if isinstance(value, datetime) and is_aware(value):
        return value
    if value:
        return make_aware(value)
    return value


def is_database_scheduler(scheduler):
    if not scheduler:
        return False
    from kombu.utils import symbol_by_name
    from .schedulers import DatabaseScheduler
    return issubclass(symbol_by_name(scheduler), DatabaseScheduler)


def fromtimestamp(value):
    if getattr(settings, 'CELERY_ENABLE_UTC', False):
        return datetime.utcfromtimestamp(value)
    else:
        return datetime.fromtimestamp(value)
